@@ -22,6 +22,7 @@ from snowflake.ml.feature_store import feature_store
 from snowflake.ml.feature_store.entity import Entity
 from snowflake.ml.lineage import lineage_node
 from snowflake.snowpark import DataFrame, Session
+from snowflake.snowpark.exceptions import SnowparkSQLException
 from snowflake.snowpark.types import (
     DateType,
     StructType,
@@ -167,6 +168,9 @@ class FeatureView(lineage_node.LineageNode):
         refresh_freq: Optional[str] = None,
         desc: str = "",
         warehouse: Optional[str] = None,
+        initialize: str = "ON_CREATE",
+        refresh_mode: str = "AUTO",
+        cluster_by: Optional[List[str]] = None,
         **_kwargs: Any,
     ) -> None:
         """
@@ -190,6 +194,16 @@ class FeatureView(lineage_node.LineageNode):
             warehouse: warehouse to refresh feature view. Not needed for static feature view (refresh_freq is None).
                 For managed feature view, this warehouse will overwrite the default warehouse of Feature Store if it is
                 specified, otherwise the default warehouse will be used.
+            initialize: Specifies the behavior of the initial refresh of feature view. This property cannot be altered
+                after you register the feature view. It supports ON_CREATE (default) or ON_SCHEDULE. ON_CREATE refreshes
+                the feature view synchronously at creation. ON_SCHEDULE refreshes the feature view at the next scheduled
+                refresh. It is only effective when refresh_freq is not None.
+            refresh_mode: The refresh mode of managed feature view. The value can be 'AUTO', 'FULL' or 'INCREMENETAL'.
+                For managed feature view, the default value is 'AUTO'. For static feature view it has no effect.
+                Check https://docs.snowflake.com/en/sql-reference/sql/create-dynamic-table for for details.
+            cluster_by: Columns to cluster the feature view by.
+                - Defaults to the join keys from entities.
+                - If `timestamp_col` is provided, it is added to the default clustering keys.
             _kwargs: reserved kwargs for system generated args. NOTE: DO NOT USE.
 
         Example::
@@ -214,6 +228,7 @@ class FeatureView(lineage_node.LineageNode):
             >>> print(registered_fv.status)
             FeatureViewStatus.ACTIVE
 
+        # noqa: DAR401
         """
 
         self._name: SqlIdentifier = SqlIdentifier(name)
@@ -223,18 +238,30 @@ class FeatureView(lineage_node.LineageNode):
             SqlIdentifier(timestamp_col) if timestamp_col is not None else None
         )
         self._desc: str = desc
-        self._infer_schema_df: DataFrame = _kwargs.get("_infer_schema_df", self._feature_df)
+        self._infer_schema_df: DataFrame = _kwargs.pop("_infer_schema_df", self._feature_df)
         self._query: str = self._get_query()
         self._version: Optional[FeatureViewVersion] = None
         self._status: FeatureViewStatus = FeatureViewStatus.DRAFT
-        self._feature_desc: OrderedDict[SqlIdentifier, str] = OrderedDict((f, "") for f in self._get_feature_names())
+        feature_names = self._get_feature_names()
+        self._feature_desc: Optional[OrderedDict[SqlIdentifier, str]] = (
+            OrderedDict((f, "") for f in feature_names) if feature_names is not None else None
+        )
         self._refresh_freq: Optional[str] = refresh_freq
         self._database: Optional[SqlIdentifier] = None
         self._schema: Optional[SqlIdentifier] = None
+        self._initialize: str = initialize
         self._warehouse: Optional[SqlIdentifier] = SqlIdentifier(warehouse) if warehouse is not None else None
-        self._refresh_mode: Optional[str] = _kwargs.get("refresh_mode", "AUTO")
+        self._refresh_mode: Optional[str] = refresh_mode
         self._refresh_mode_reason: Optional[str] = None
         self._owner: Optional[str] = None
+        self._cluster_by: List[SqlIdentifier] = (
+            [SqlIdentifier(col) for col in cluster_by] if cluster_by is not None else self._get_default_cluster_by()
+        )
+
+        # Validate kwargs
+        if _kwargs:
+            raise TypeError(f"FeatureView.__init__ got an unexpected keyword argument: '{next(iter(_kwargs.keys()))}'")
+
         self._validate()
 
     def slice(self, names: List[str]) -> FeatureViewSlice:
@@ -345,6 +372,15 @@ class FeatureView(lineage_node.LineageNode):
                 ('START_STATION_LATITUDE', 'Latitude of the start station.')])
 
         """
+        if self._feature_desc is None:
+            warnings.warn(
+                "Failed to read feature view schema. Probably feature view is not refreshed yet. "
+                "Schema will be available after initial refresh.",
+                stacklevel=2,
+                category=UserWarning,
+            )
+            return self
+
         for f, d in descs.items():
             f = SqlIdentifier(f)
             if f not in self._feature_desc:
@@ -371,6 +407,10 @@ class FeatureView(lineage_node.LineageNode):
     def timestamp_col(self) -> Optional[SqlIdentifier]:
         return self._timestamp_col
 
+    @property
+    def cluster_by(self) -> Optional[List[SqlIdentifier]]:
+        return self._cluster_by
+
     @property
     def desc(self) -> str:
         return self._desc
@@ -424,10 +464,10 @@ class FeatureView(lineage_node.LineageNode):
 
     @property
     def feature_names(self) -> List[SqlIdentifier]:
-        return list(self._feature_desc.keys())
+        return list(self._feature_desc.keys()) if self._feature_desc is not None else []
 
     @property
-    def feature_descs(self) -> Dict[SqlIdentifier, str]:
+    def feature_descs(self) -> Optional[Dict[SqlIdentifier, str]]:
         return self._feature_desc
 
     def list_columns(self) -> DataFrame:
@@ -463,7 +503,17 @@ class FeatureView(lineage_node.LineageNode):
 
         """
         session = self._feature_df.session
-        rows = []
+        rows = []  # type: ignore[var-annotated]
+
+        if self.feature_descs is None:
+            warnings.warn(
+                "Failed to read feature view schema. Probably feature view is not refreshed yet. "
+                "Schema will be available after initial refresh.",
+                stacklevel=2,
+                category=UserWarning,
+            )
+            return session.create_dataframe(rows, schema=["name", "category", "dtype", "desc"])
+
         for name, type in self._feature_df.dtypes:
             if SqlIdentifier(name) in self.feature_descs:
                 desc = self.feature_descs[SqlIdentifier(name)]
@@ -565,6 +615,10 @@ class FeatureView(lineage_node.LineageNode):
         )
         self._warehouse = SqlIdentifier(new_value)
 
+    @property
+    def initialize(self) -> str:
+        return self._initialize
+
     @property
     def output_schema(self) -> StructType:
         return self._infer_schema_df.schema
@@ -601,33 +655,57 @@ Got {len(self._feature_df.queries['queries'])}: {self._feature_df.queries['queri
                 f"FeatureView name `{self._name}` contains invalid character `{_FEATURE_VIEW_NAME_DELIMITER}`."
             )
 
-        unescaped_df_cols = to_sql_identifiers(self._infer_schema_df.columns)
-        for e in self._entities:
-            for k in e.join_keys:
-                if k not in unescaped_df_cols:
-                    raise ValueError(
-                        f"join_key {k} in Entity {e.name} is not found in input dataframe: {unescaped_df_cols}"
-                    )
+        df_cols = self._get_column_names()
+        if df_cols is not None:
+            for e in self._entities:
+                for k in e.join_keys:
+                    if k not in df_cols:
+                        raise ValueError(f"join_key {k} in Entity {e.name} is not found in input dataframe: {df_cols}")
 
-        if self._timestamp_col is not None:
-            ts_col = self._timestamp_col
-            if ts_col == SqlIdentifier(_TIMESTAMP_COL_PLACEHOLDER):
-                raise ValueError(f"Invalid timestamp_col name, cannot be {_TIMESTAMP_COL_PLACEHOLDER}.")
-            if ts_col not in to_sql_identifiers(self._infer_schema_df.columns):
-                raise ValueError(f"timestamp_col {ts_col} is not found in input dataframe.")
+            if self._timestamp_col is not None:
+                ts_col = self._timestamp_col
+                if ts_col == SqlIdentifier(_TIMESTAMP_COL_PLACEHOLDER):
+                    raise ValueError(f"Invalid timestamp_col name, cannot be {_TIMESTAMP_COL_PLACEHOLDER}.")
+                if ts_col not in df_cols:
+                    raise ValueError(f"timestamp_col {ts_col} is not found in input dataframe.")
 
-            col_type = self._infer_schema_df.schema[ts_col].datatype
-            if not isinstance(col_type, (DateType, TimeType, TimestampType, _NumericType)):
-                raise ValueError(f"Invalid data type for timestamp_col {ts_col}: {col_type}.")
+                col_type = self._infer_schema_df.schema[ts_col].datatype
+                if not isinstance(col_type, (DateType, TimeType, TimestampType, _NumericType)):
+                    raise ValueError(f"Invalid data type for timestamp_col {ts_col}: {col_type}.")
+
+            if self.cluster_by is not None:
+                for column in self.cluster_by:
+                    if column not in df_cols:
+                        raise ValueError(
+                            f"Column '{column}' in `cluster_by` is not in the feature DataFrame schema. "
+                            f"{df_cols}, {self.cluster_by}"
+                        )
 
         if re.match(_RESULT_SCAN_QUERY_PATTERN, self._query) is not None:
             raise ValueError(f"feature_df should not be reading from RESULT_SCAN. Invalid query: {self._query}")
 
-    def _get_feature_names(self) -> List[SqlIdentifier]:
+        if self._initialize not in ["ON_CREATE", "ON_SCHEDULE"]:
+            raise ValueError("'initialize' only supports ON_CREATE or ON_SCHEDULE.")
+
+    def _get_column_names(self) -> Optional[List[SqlIdentifier]]:
+        try:
+            return to_sql_identifiers(self._infer_schema_df.columns)
+        except SnowparkSQLException as e:
+            warnings.warn(
+                "Failed to read feature view schema. Probably feature view is not refreshed yet. "
+                f"Schema will be available after initial refresh. Original exception: {e}",
+                stacklevel=2,
+                category=UserWarning,
+            )
+            return None
+
+    def _get_feature_names(self) -> Optional[List[SqlIdentifier]]:
         join_keys = [k for e in self._entities for k in e.join_keys]
         ts_col = [self._timestamp_col] if self._timestamp_col is not None else []
-        feature_names = to_sql_identifiers(self._infer_schema_df.columns, case_sensitive=False)
-        return [c for c in feature_names if c not in join_keys + ts_col]
+        feature_names = self._get_column_names()
+        if feature_names is not None:
+            return [c for c in feature_names if c not in join_keys + ts_col]
+        return None
 
     def __repr__(self) -> str:
         states = (f"{k}={v}" for k, v in vars(self).items())
@@ -670,11 +748,13 @@ Got {len(self._feature_df.queries['queries'])}: {self._feature_df.queries['queri
         fv_dict["_schema"] = str(self._schema) if self._schema is not None else None
         fv_dict["_warehouse"] = str(self._warehouse) if self._warehouse is not None else None
         fv_dict["_timestamp_col"] = str(self._timestamp_col) if self._timestamp_col is not None else None
+        fv_dict["_initialize"] = str(self._initialize)
 
         feature_desc_dict = {}
-        for k, v in self._feature_desc.items():
-            feature_desc_dict[k.identifier()] = v
-        fv_dict["_feature_desc"] = feature_desc_dict
+        if self._feature_desc is not None:
+            for k, v in self._feature_desc.items():
+                feature_desc_dict[k.identifier()] = v
+            fv_dict["_feature_desc"] = feature_desc_dict
 
         lineage_node_keys = [key for key in fv_dict if key.startswith("_node") or key == "_session"]
 
@@ -760,6 +840,7 @@ Got {len(self._feature_df.queries['queries'])}: {self._feature_df.queries['queri
             warehouse=json_dict["_warehouse"],
             refresh_mode=json_dict["_refresh_mode"],
             refresh_mode_reason=json_dict["_refresh_mode_reason"],
+            initialize=json_dict["_initialize"],
             owner=json_dict["_owner"],
             infer_schema_df=session.sql(json_dict.get("_infer_schema_query", None)),
             session=session,
@@ -830,9 +911,11 @@ Got {len(self._feature_df.queries['queries'])}: {self._feature_df.queries['queri
         warehouse: Optional[str],
         refresh_mode: Optional[str],
         refresh_mode_reason: Optional[str],
+        initialize: str,
         owner: Optional[str],
         infer_schema_df: Optional[DataFrame],
         session: Session,
+        cluster_by: Optional[List[str]] = None,
     ) -> FeatureView:
         fv = FeatureView(
             name=name,
@@ -841,6 +924,7 @@ Got {len(self._feature_df.queries['queries'])}: {self._feature_df.queries['queri
             timestamp_col=timestamp_col,
             desc=desc,
             _infer_schema_df=infer_schema_df,
+            cluster_by=cluster_by,
         )
         fv._version = FeatureViewVersion(version) if version is not None else None
         fv._status = status
@@ -850,6 +934,7 @@ Got {len(self._feature_df.queries['queries'])}: {self._feature_df.queries['queri
         fv._warehouse = SqlIdentifier(warehouse) if warehouse is not None else None
         fv._refresh_mode = refresh_mode
         fv._refresh_mode_reason = refresh_mode_reason
+        fv._initialize = initialize
         fv._owner = owner
         fv.attach_feature_desc(feature_descs)
 
@@ -858,5 +943,23 @@ Got {len(self._feature_df.queries['queries'])}: {self._feature_df.queries['queri
         )
         return fv
 
+    #
+    def _get_default_cluster_by(self) -> List[SqlIdentifier]:
+        """
+        Get default columns to cluster the feature view by.
+        Default cluster_by columns are join keys from entities and timestamp_col if it exists
+
+        Returns:
+            List of SqlIdentifiers representing the default columns to cluster the feature view by.
+        """
+        # We don't focus on the order of entities here, as users can define a custom 'cluster_by'
+        # if a specific order is required.
+        default_cluster_by_cols = [key for entity in self.entities if entity.join_keys for key in entity.join_keys]
+
+        if self.timestamp_col:
+            default_cluster_by_cols.append(self.timestamp_col)
+
+        return default_cluster_by_cols
+
 
 lineage_node.DOMAIN_LINEAGE_REGISTRY["feature_view"] = FeatureView
